/*
Copyright (c) Uber Technologies, Inc.

This source code is licensed under the MIT license found in the
LICENSE file in the root directory of this source tree.
*/
import { styled } from '../styles/index.js';
import { ACTION_POSITION } from './constants.js';
export var StyledRoot = styled('div', function (_ref) {
  var $theme = _ref.$theme,
      $backgroundColor = _ref.$backgroundColor,
      $color = _ref.$color,
      $nested = _ref.$nested;
  var radius = $nested ? $theme.borders.radius300 : $theme.borders.radius400;
  return {
    backgroundColor: $backgroundColor,
    borderStartStartRadius: radius,
    borderStartEndRadius: radius,
    borderEndStartRadius: radius,
    borderEndEndRadius: radius,
    color: $color,
    display: 'grid',
    gridColumnGap: $theme.sizing.scale600,
    gridTemplateColumns: 'min-content auto min-content',
    gridTemplateRows: 'auto min-content',
    margin: $theme.sizing.scale600
  };
});
StyledRoot.displayName = "StyledRoot";
export var StyledLeadingContent = styled('div', function (_ref2) {
  var $theme = _ref2.$theme,
      $includesArtwork = _ref2.$includesArtwork;
  return {
    alignItems: 'center',
    display: 'flex',
    paddingInlineStart: $includesArtwork ? $theme.sizing.scale600 : null
  };
});
StyledLeadingContent.displayName = "StyledLeadingContent";
export var StyledMessageContent = styled('div', function (_ref3) {
  var $theme = _ref3.$theme,
      $actionPosition = _ref3.$actionPosition;
  return {
    display: 'flex',
    flexDirection: 'column',
    justifyContent: 'center',
    paddingBlockStart: $theme.sizing.scale600,
    paddingBlockEnd: $actionPosition === ACTION_POSITION.trailing ? $theme.sizing.scale600 : null
  };
});
StyledMessageContent.displayName = "StyledMessageContent";
export var StyledTrailingContent = styled('div', function (_ref4) {
  var $theme = _ref4.$theme;
  return {
    display: 'flex',
    gridRowEnd: 'span 2',
    marginInlineStart: 'auto'
  };
});
StyledTrailingContent.displayName = "StyledTrailingContent";
export var StyledBelowContent = styled('div', function (_ref5) {
  var $theme = _ref5.$theme,
      $actionPosition = _ref5.$actionPosition;
  return {
    gridColumnStart: 2,
    paddingBlockStart: $actionPosition === ACTION_POSITION.below ? $theme.sizing.scale300 : null,
    paddingBlockEnd: $actionPosition === ACTION_POSITION.below ? $theme.sizing.scale600 : null
  };
});
StyledBelowContent.displayName = "StyledBelowContent";
export var StyledTitle = styled('div', function (_ref6) {
  var $theme = _ref6.$theme;
  return $theme.typography.LabelMedium;
});
StyledTitle.displayName = "StyledTitle";
export var StyledMessage = styled('div', function (_ref7) {
  var $theme = _ref7.$theme;
  return $theme.typography.ParagraphMedium;
});
StyledMessage.displayName = "StyledMessage";
export var StyledTrailingButtonContainer = styled('div', function (_ref8) {
  var $theme = _ref8.$theme;
  return {
    display: 'flex',
    alignItems: 'center',
    paddingInlineEnd: $theme.sizing.scale600
  };
});
StyledTrailingButtonContainer.displayName = "StyledTrailingButtonContainer";
export var StyledTrailingIconButton = styled('button', function (_ref9) {
  var $theme = _ref9.$theme,
      $nested = _ref9.$nested;
  var radius = $nested ? $theme.borders.radius300 : $theme.borders.radius400;
  return {
    alignItems: 'center',
    background: 'none',
    border: 'none',
    borderStartEndRadius: radius,
    borderEndEndRadius: radius,
    color: 'inherit',
    cursor: 'pointer',
    display: 'flex',
    paddingInlineStart: $theme.sizing.scale600,
    paddingInlineEnd: $theme.sizing.scale600,
    ':hover': {
      boxShadow: 'inset 999px 999px 0px rgba(0, 0, 0, 0.04)'
    },
    ':active': {
      boxShadow: 'inset 999px 999px 0px rgba(0, 0, 0, 0.08)'
    }
  };
});
StyledTrailingIconButton.displayName = "StyledTrailingIconButton";